/*
 * Copyright (C) 2013 by the University of Southern California
 * $Id$
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License,
 * version 2, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 */
#ifndef DNS_HEADER_H
#define DNS_HEADER_H 1

#define ZBIT_OFFSET 8
#define UDP_ANS_SIZE 4096
#define DNS_MESSAGEEXTFLAG_DO  0x8000U
#define DNS_MESSAGEEXTFLAG_TLS 0x4000U

//DNS header structure
struct DNS_HEADER
{
    unsigned short id; // identification number

    unsigned char rd :1; // recursion desired
    unsigned char tc :1; // truncated message
    unsigned char aa :1; // authoritive answer
    unsigned char opcode :4; // purpose of message
    unsigned char qr :1; // query/response flag

    unsigned char rcode :4; // response code
    unsigned char cd :1; // checking disabled
    unsigned char ad :1; // authenticated data
    unsigned char z :1; // its z! reserved
    unsigned char ra :1; // recursion available

    unsigned short q_count; // number of question entries
    unsigned short ans_count; // number of answer entries
    unsigned short auth_count; // number of authority entries
    unsigned short add_count; // number of resource entries
};



// EDNS Header structure
struct EDNS_HEADER
{
  unsigned char  name;
  unsigned short type;
  unsigned short udp_payload;
  unsigned char  rcode;
  unsigned char  version;
  unsigned short Z;
  unsigned short len;
} __attribute__ ((packed));



//Constant sized fields of query structure
struct QUESTION
{
    unsigned short qtype;
    unsigned short qclass;
};

//Constant sized fields of the resource record structure
#pragma pack(push, 1)
struct R_DATA
{
    unsigned short type;
    unsigned short _class;
    union
    {
        struct{
            unsigned char _ext_rcode; //EDNS extended RCODE
            unsigned char _version; //EDNS version
            unsigned char _do :1;   //do bit
            unsigned char _tls :1;  //tls bit
            unsigned char _rest_flags :6;
            unsigned char _reserved_flags;  //rest flags are all reserved.
        } _edns_ttl;
        unsigned int _ttl;
    };
    unsigned short data_len;
};
#pragma pack(pop)

//Pointers to resource record contents
struct RES_RECORD
{
    unsigned char *name;
    struct R_DATA *resource;
    unsigned char *rdata;
};

//Pointers to resource record contents
struct EDNS_DATA 
{
    char name;
    char type[2];
    char udp_len[2];
};


#endif
